const uuid = require('uuid')
const { pick, get, set, has } = require('lodash')
const config = require('config')
const { v4 } = require('uuid')

const {
  services,
  authsome: authsomeHelper,
  Collection,
} = require('pubsweet-component-helper-service')

const s3Config = get(config, 'pubsweet-component-aws-s3', {})
const mtsConfig = get(config, 'mts-service', {})
const MTSService = require('pubsweet-component-mts-package')

const notifications = require('./notifications/notifications')

module.exports = models => async (req, res) => {
  const { recommendation, comments, recommendationType } = req.body
  if (!services.checkForUndefinedParams(recommendationType))
    return res.status(400).json({ error: 'Recommendation type is required.' })

  const reqUser = await models.User.find(req.user)
  const isEditorInChief = reqUser.editorInChief || reqUser.admin

  const { collectionId, fragmentId } = req.params

  let collection, fragment

  try {
    collection = await models.Collection.find(collectionId)
    if (!collection.fragments.includes(fragmentId))
      return res.status(400).json({
        error: `Collection and fragment do not match.`,
      })
    fragment = await models.Fragment.find(fragmentId)
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'Item')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }

  const authsome = authsomeHelper.getAuthsome(models)
  const target = {
    fragment,
    path: req.route.path,
  }
  const canPost = await authsome.can(req.user, 'POST', target)
  if (!canPost)
    return res.status(403).json({
      error: 'Unauthorized.',
    })

  fragment.recommendations = fragment.recommendations || []
  const newRecommendation = {
    id: uuid.v4(),
    userId: reqUser.id,
    createdOn: Date.now(),
    updatedOn: Date.now(),
    recommendationType,
  }

  newRecommendation.recommendation = recommendation || undefined
  newRecommendation.comments = comments || undefined

  if (recommendationType === 'editorRecommendation') {
    const collectionHelper = new Collection({ collection })

    collectionHelper.updateStatusOnRecommendation({
      isEditorInChief,
      recommendation,
    })

    if (!isEditorInChief && ['minor', 'major'].includes(recommendation)) {
      fragment.revision = pick(fragment, ['authors', 'files', 'metadata'])
    }

    const technicalChecks = get(collection, 'technicalChecks', {})
    const { hasEQA } = technicalChecks

    if (isEditorInChief && recommendation === 'publish' && !hasEQA) {
      const { journal, xmlParser, ftp } = mtsConfig
      const MTS = new MTSService(journal, xmlParser, s3Config, ftp)
      const packageFragment = {
        ...fragment,
        metadata: {
          ...fragment.metadata,
          customId: collection.customId,
        },
      }

      await MTS.sendPackage({ fragment: packageFragment, isEQA: true })

      collection.status = 'inQA'
      set(collection, 'technicalChecks.token', v4())
      set(collection, 'technicalChecks.hasEQA', false)
      await collection.save()
    }

    /* if the EiC returns the manuscript to the HE after the EQA has been performed
       then remove all properties from the technicalChecks property so that the manuscript
       can go through the EQA process again
    */
    if (
      isEditorInChief &&
      recommendation === 'return-to-handling-editor' &&
      has(collection.technicalChecks, 'hasEQA')
    ) {
      collection.technicalChecks = {}
      await collection.save()
    }

    notifications.sendNotifications({
      hasEQA,
      fragment,
      collection,
      isEditorInChief,
      newRecommendation,
      UserModel: models.User,
      baseUrl: services.getBaseUrl(req),
    })
  }

  fragment.recommendations.push(newRecommendation)
  fragment.save()

  return res.status(200).json(newRecommendation)
}
